"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Log_1 = require("./Log");
const Diff_1 = require("./Diff");
const CommitBuilder_1 = require("./CommitBuilder");
/**
 *  Asynchronously fetches the metadata of all commits within a particular reference range.
 *  @param dir The path to the root directory of a git repository.
 *  @param startRef A reference string, such as a commit hash or tag name, which designates the
 *    beginning of the range (exclusive.) If not defined, all ancestors of endRef will be listed.
 *  @param endRef A reference string, such as a commit hash or tag name, which designates the end
 *    of the range (inclusive.) If not defined, endRef will be assumed to be 'HEAD'.
 *  @param includeDiff An optional boolean which, if set to true, will cause information on the
 *    files touched by each commit to also be collected. This functionality is disabled by default
 *    for performance reasons.
 *  @return A Promise for an array of objects containing the metadata of each commit in the range.
 */
function gitLog(dir = process.cwd(), startRef, endRef, includeKeys) {
    return __awaiter(this, void 0, void 0, function* () {
        const { commitBuilder, includeDiff, rangeString } = prepare(startRef, endRef, includeKeys);
        const data = yield Log_1.Log.fetch(rangeString, commitBuilder.formatString, dir);
        const log = commitBuilder.buildAll(data);
        if (includeDiff) {
            const applyDiff = (commit) => Diff_1.Diff.make(commit, dir).then(diff => Object.assign(commit, { diff }));
            return Promise.all(log.map(applyDiff));
        }
        return log;
    });
}
exports.gitLog = gitLog;
/**
 *  Synchronously fetches the metadata of all commits within a particular reference range.
 *  @param dir The path to the root directory of a git repository.
 *  @param startRef A reference string, such as a commit hash or tag name, which designates the
 *    beginning of the range (exclusive.) If not defined, all ancestors of endRef will be listed.
 *  @param endRef A reference string, such as a commit hash or tag name, which designates the end
 *    of the range (inclusive.) If not defined, endRef will be assumed to be 'HEAD'.
 *  @param includeDiff An optional boolean which, if set to true, will cause information on the
 *    files touched by each commit to also be collected. This functionality is disabled by default
 *    for performance reasons.
 *  @return An array of objects containing the metadata of each commit in the range.
 */
function gitLogSync(dir = process.cwd(), startRef, endRef, includeKeys) {
    const { commitBuilder, includeDiff, rangeString } = prepare(startRef, endRef, includeKeys);
    const data = Log_1.Log.fetchSync(rangeString, commitBuilder.formatString, dir);
    const log = commitBuilder.buildAll(data);
    if (includeDiff) {
        for (const commit of log) {
            const diff = Diff_1.Diff.makeSync(commit, dir);
            Object.assign(commit, { diff });
        }
    }
    return log;
}
exports.gitLogSync = gitLogSync;
function prepare(startRef, endRef, includeKeys) {
    const optionalKeys = new Set(includeKeys);
    const includeDiff = optionalKeys.delete('diff');
    const commitBuilder = new CommitBuilder_1.CommitBuilder(optionalKeys);
    const rangeString = getRangeString(startRef, endRef);
    return { commitBuilder, includeDiff, rangeString };
}
function getRangeString(startRef, endRef = 'HEAD') {
    if (startRef !== undefined) {
        return `${startRef}..${endRef}`;
    }
    return endRef;
}
